/*
  Originally written by Naoto Kaneko
  Last updated: 3.Jul.2010, buggy-and-messy version
  Latest info: http://www37.atwiki.jp/pjkyt/
 */

package com.kyt.android.view;

import android.graphics.*;

/**
 * Base node of the drawing object to be used in the ViewportedView.
 * 
 * It assumes a a very simple graphic object having a square draw area.  
 *  And X and Y are the center of the obejct.  This can be overridden 
 *  in 
 *  
 * Using this object together with DrawNodeList, it becomes possible to
 *  create the list of nodes only that are inside the window (getNodesToDraw()).
 *  This is so to eliminate the drawing of nodes that won't anyway appear
 *  in the view port.
 */
abstract public class DrawNode {
	private String Name;
	private int X,Y; // center of the node
	private int Radius;
	
	public DrawNode(String name, int x, int y, int radius) {
		Name = name;
		X = x;
		Y = y;
		Radius = radius;
	}
	
	public String getName() { return Name;}
	public int getX() { return X;}
	public int getY() { return Y;}
	public int getRadius() { return Radius;}
	
	/**
	 * These getXXXBound() function must be overridden based on the shape 
	 *  of the derived node.  For example, it is a node representing a string
	 *  they will be determined with the text length and font metric.
	 * Those functions are used by (1) pointIsInside(), which will be used in the 
	 *  tap detection of the node in the view port, and (2) getNodesToDraw() of 
	 *  the DrawNode.  So their proper implementation is crucial.
	 */
	public int getLeftBound() {
		return X-Radius;
	}
	
	public int getRightBound() {
		return X+Radius;
	}
	
	public int getTopBound() {
		return Y-Radius;
	}
	
	public int getBottomBound() {
		return Y+Radius;
	}
	
	public void setName(String name) { Name = name;}
	public void setX(int x) { X = x;}
	public void setY(int y) { Y = y;}
	public void setRadius(int radius) { Radius = radius;}
	
	/**
	 * Returns true is given (x,y) falls inside the area of this node
	 */
	public boolean pointIsInside(int x, int y) {
		if(x > getLeftBound() && x < getRightBound() 
				&& y > getTopBound() && y < getBottomBound() ) {
			return true;
		}
		else {
			return false;
		}
	}
	
	abstract public void draw(Canvas canvas, Paint p, DrawWindow window);
}
