/*
  Originally written by Naoto Kaneko
  Last updated: 3.Jul.2010, buggy-and-messy version
  Latest info: http://www37.atwiki.jp/pjkyt/
 */

package com.kyt.android.view;

import android.content.*;
import android.view.*;

public class DrawWindow {
	// How large the the world coordinate is; the world is square
	private final int GroundPlaneSize;
	
	// Shorter end of the window must be at least as large as this size
	private final int MinWindowSize = 10;
	
	private int ViewPortWidth, ViewPortHeight; // on full-screen view, they will be screen width and height
	private float AspectRatio; // longer_side = AspectRatio * shorter_side 
	private float ZoomFactor; // the larger the factor is, the more zoomed in the window is
	
	// X,Y sets the upper left corner of the window in the virtual ground plane
	// On the absolute screen, this point becomes (0,0)
	private int X,Y; 
	
	// WindowWidth, WindowHeight sets the size of the window in the ground plane
	private int WindowWidth, WindowHeight;
	
	public DrawWindow(int groundPlaneSize, Context context) {
		// get screen size from the system
		updateViewPortSize(context);

		if(groundPlaneSize < ViewPortWidth || groundPlaneSize < ViewPortHeight) {
			// the ground plane must be larger than the view port
			GroundPlaneSize = (ViewPortWidth > ViewPortHeight) ? ViewPortWidth : ViewPortHeight;
		}
		else {
			GroundPlaneSize = groundPlaneSize;
		}
		
		// locate the window in the middle of the ground plane 
		X = (GroundPlaneSize - ViewPortWidth) / 2;
		Y = (GroundPlaneSize - ViewPortHeight) / 2;
		
		if(ViewPortWidth < ViewPortHeight) { // portrait
			AspectRatio = (float)ViewPortHeight / (float)ViewPortWidth;
		}
		else { // landscape
			AspectRatio = (float)ViewPortWidth / (float)ViewPortHeight;
		}
		setZoomFactor(1.0f);
	}
	
	// -----------
	
	public int getX() { return X;}
	public int getY() {	return Y;}
	public int getWidth() { return WindowWidth;}
	public int getHeight() { return WindowHeight;}
	public float getZoomFactor() { return ZoomFactor;}
	public int getViewPortWidth() { return ViewPortWidth;}
	public int getViewPortHeight() { return ViewPortHeight;}
	public int getGroundPlaneSize() { return GroundPlaneSize;}
	
	// coord translation: the virtual ground plane -> view port
	public int toViewPortX(int x) {
		return (int)((x - X)*ZoomFactor + 0.5);
	}
	
	public int toViewPortY(int y) {
		return (int)((y - Y)*ZoomFactor + 0.5);
	}
	
	public int toViewPortMetric(int length) {
		return (int)(length*ZoomFactor + 0.5);
	}
	
	// coord translation: the view port -> virtual ground plane
	public int toGroundPlaneX(int x) {
		return (int)(x/ZoomFactor + 0.5) + X;
	}

	public int toGroundPlaneY(int y) {
		return (int)(y/ZoomFactor + 0.5) + Y;		
	}
	
	public int toGroundPlaneMetric(int length) {
		return (int)(length/ZoomFactor + 0.5);	
	}
	// -----------
	
	public void updateViewPortSize(Context context) {
		Display display = ((WindowManager) context.getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay();
		ViewPortWidth = display.getWidth(); 
		ViewPortHeight = display.getHeight();	
	}
	
	// set x-y of the upper-left corner of the window; it is the origin 
	public void setXY(int x, int y) {
		X = x;
		Y = y;
		correctXY();
	}
	
	public void setX(int x) {
		X = x;
		correctXY();
	}
	
	public void setY(int y) {
		Y = y;
		correctXY();
	}
	
	public void setZoomFactor(float factor) {
		if(ViewPortWidth < ViewPortHeight) { // portrait
			setSize((int)(ViewPortWidth / factor));
		}
		else { // landscape
			setSize((int)(ViewPortHeight / factor));
		}
	}
	
	public void setSize(int shorterSide) {
		int shorter_side = shorterSide;
		
		if(shorter_side < MinWindowSize) {
			shorter_side = MinWindowSize;
		}
		int longer_side = (int)(shorter_side * AspectRatio);
	
		// make sure the window goes no larger than the virtual ground plane 
		if(longer_side > GroundPlaneSize) {
			longer_side = GroundPlaneSize;
			shorter_side = (int)(longer_side / AspectRatio);
		}
		
		// update x,y,width,height
		if(ViewPortWidth < ViewPortHeight) { // portrait
			X += (WindowWidth - shorter_side) / 2; // (old_width - new_width) / 2
			Y += (WindowHeight - longer_side) / 2;
			
			WindowWidth = shorter_side;
			WindowHeight = longer_side;
		}
		else { // landscape
			X += (WindowWidth - longer_side) / 2;
			Y += (WindowHeight - shorter_side) / 2;
			
			WindowWidth = longer_side;
			WindowHeight = shorter_side;
		}
		
		ZoomFactor = (float)ViewPortWidth / (float)WindowWidth;
		correctXY();
	}
	
	// make sure the window stays inside the virtual ground plane 
	private void correctXY() {
		if(X < 0) {
			X = 0;
		}
		else if(X+WindowWidth > GroundPlaneSize) {
			X = GroundPlaneSize - WindowWidth;
		}
		
		if(Y < 0) {
			Y = 0;
		}
		else if(Y+WindowHeight > GroundPlaneSize) {
			Y = GroundPlaneSize - WindowHeight;
		}
	}
}